var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ViewChild, HostBinding, Input, HostListener, EventEmitter, Output, ElementRef, TemplateRef, Renderer2 } from "@angular/core";
import { Util } from "../../../misc/util/index";
import { DropdownService } from "../../dropdown/services/dropdown.service";
import { TuiDropdownMenu } from "../../dropdown/directives/dropdown-menu";
import { TuiLocalizationService } from "../../../behaviors/localization/services/localization.service";
import { TuiTagsInputService } from "../services/tags-input.service";
var TuiTagsInput = (function () {
    function TuiTagsInput(_element, renderer, _localizationService) {
        var _this = this;
        this._element = _element;
        this._localizationService = _localizationService;
        this.dropdownService = new DropdownService();
        this.searchService = new TuiTagsInputService();
        this.selectedOptions = [];
        this._delimiter = ',';
        this.onLocaleUpdate();
        this._localizationService.onLanguageUpdate.subscribe(function () { return _this.onLocaleUpdate(); });
        this._searchClasses = true;
        this.hasIcon = true;
        this.retainSelectedResult = true;
        this.searchDelay = 200;
        this.maxResults = 7;
        this.minSearchLength = 1;
        this.maxTagsCount = 10;
        this.onResultSelected = new EventEmitter();
        this.transition = "scale";
        this.transitionDuration = 200;
        this._documentClickListener = renderer.listen("document", "click", function (e) { return _this.onDocumentClick(e); });
    }
    Object.defineProperty(TuiTagsInput.prototype, "isActive", {
        get: function () {
            return this.dropdownService.isOpen;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "placeholder", {
        // Gets & sets the placeholder text displayed inside the text input.
        get: function () {
            return this._placeholder || this.localeValues.placeholder;
        },
        set: function (placeholder) {
            this._placeholder = placeholder;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "localeValues", {
        get: function () {
            return this._localizationService.override(this._localeValues, this.localeOverrides);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "query", {
        get: function () {
            return this.searchService.query;
        },
        set: function (query) {
            var lastDelimiterIndex = query.lastIndexOf(this._delimiter);
            if (query.indexOf(this._delimiter) !== -1) {
                this.createTags(query);
            }
            this.selectedResult = undefined;
            this.searchService.query = query.slice(lastDelimiterIndex + 1);
            this.open();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "options", {
        set: function (options) {
            if (options) {
                this.searchService.options = options;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "optionsFilter", {
        set: function (filter) {
            if (filter) {
                this.searchService.optionsFilter = filter;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "optionsLookup", {
        set: function (lookupFn) {
            this.searchService.optionsLookup = lookupFn;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "optionsField", {
        get: function () {
            return this.searchService.optionsField;
        },
        set: function (field) {
            this.searchService.optionsField = field;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "resultFormatter", {
        get: function () {
            var _this = this;
            if (this._resultFormatter) {
                return this._resultFormatter;
            }
            else if (this.searchService.optionsLookup) {
                return function (r) { return _this.readValue(r); };
            }
            else {
                return function (r, q) { return _this.searchService.highlightMatches(_this.readValue(r), q); };
            }
        },
        set: function (formatter) {
            this._resultFormatter = formatter;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "configuredFormatter", {
        get: function () {
            var _this = this;
            if (this.searchService.optionsLookup) {
                return function (o) { return _this.labelGetter(o); };
            }
            else {
                return function (o) { return _this.searchService.highlightMatches(_this.labelGetter(o), _this.query || ""); };
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "labelGetter", {
        get: function () {
            var _this = this;
            // Helper function to retrieve the label from an item.
            return function (obj) {
                var label = Util.Object.readValue(obj, _this.optionsField);
                if (label != undefined) {
                    return label.toString();
                }
                return "";
            };
        },
        enumerable: true,
        configurable: true
    });
    TuiTagsInput.prototype.createTags = function (query) {
        var _this = this;
        var field = this.optionsField || '';
        var tags = query.split(this._delimiter);
        tags.splice(-1, 1);
        tags.forEach(function (tag) {
            if (tag.length > 0) {
                var tagObject = {};
                tagObject[field] = tag;
                if (_this.selectedOptions.length < _this.maxTagsCount) {
                    _this.selectedOptions.push(tagObject);
                }
            }
        });
    };
    Object.defineProperty(TuiTagsInput.prototype, "searchDelay", {
        set: function (delay) {
            this.searchService.searchDelay = delay;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "isSearching", {
        get: function () {
            return this.searchService.isSearching;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TuiTagsInput.prototype, "results", {
        get: function () {
            var _this = this;
            return this.searchService.results
                .filter(function (o) { return o[_this.optionsField ? _this.optionsField : ''].slice(0, _this.query.length).toLowerCase() === _this.query.toLowerCase(); })
                .slice(0, this.maxResults);
        },
        enumerable: true,
        configurable: true
    });
    TuiTagsInput.prototype.ngAfterViewInit = function () {
        this._menu.service = this.dropdownService;
        this.searchService.getData();
    };
    TuiTagsInput.prototype.onLocaleUpdate = function () {
        this._localeValues = this._localizationService.get().search;
    };
    TuiTagsInput.prototype.select = function (result) {
        if (this.selectedOptions.length < this.maxTagsCount) {
            var selectedIndex = this.searchService.results.indexOf(result);
            this.searchService.results.splice(selectedIndex, 1);
            this.onResultSelected.emit(result);
            this.selectedOptions.push(result);
            if (this.tagSelectedCb) {
                this.tagSelectedCb.call(undefined, result);
            }
        }
        this.query = '';
    };
    TuiTagsInput.prototype.deselectOption = function (option) {
        // Update selected options to the previously selected options \ {option}.
        this.selectedOptions = this.selectedOptions.filter(function (so) { return so !== option; });
        this.searchService.results.push(option);
        if (this.tagDeselectedCb) {
            this.tagDeselectedCb.call(undefined, option);
        }
    };
    TuiTagsInput.prototype.onClick = function (e) {
        this.open();
    };
    TuiTagsInput.prototype.onFocusIn = function () {
        if (!this.dropdownService.isAnimating) {
            this.open();
        }
    };
    TuiTagsInput.prototype.onPaste = function (e) {
        e.preventDefault();
        var content = e.clipboardData.getData('text/plain');
        this.createTags(content);
    };
    TuiTagsInput.prototype.open = function () {
        if (this.searchService.query.length >= this.minSearchLength) {
            // Only open on click when there is a query entered.
            this.dropdownService.setOpenState(true);
        }
    };
    TuiTagsInput.prototype.onFocusOut = function (e) {
        if (!this._element.nativeElement.contains(e.relatedTarget)) {
            this.dropdownService.setOpenState(false);
        }
    };
    TuiTagsInput.prototype.onDocumentClick = function (e) {
        if (!this._element.nativeElement.contains(e.target)) {
            this.dropdownService.setOpenState(false);
        }
    };
    // Reads the specified field from an item.
    TuiTagsInput.prototype.readValue = function (object) {
        return Util.Object.readValue(object, this.searchService.optionsField);
    };
    TuiTagsInput.prototype.ngOnDestroy = function () {
        this._documentClickListener();
    };
    return TuiTagsInput;
}());
__decorate([
    Input(),
    __metadata("design:type", Array)
], TuiTagsInput.prototype, "selectedOptions", void 0);
__decorate([
    ViewChild(TuiDropdownMenu),
    __metadata("design:type", TuiDropdownMenu)
], TuiTagsInput.prototype, "_menu", void 0);
__decorate([
    HostBinding("class.ui"),
    HostBinding("class.search"),
    __metadata("design:type", Boolean)
], TuiTagsInput.prototype, "_searchClasses", void 0);
__decorate([
    HostBinding("class.active"),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TuiTagsInput.prototype, "isActive", null);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], TuiTagsInput.prototype, "hasIcon", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], TuiTagsInput.prototype, "minSearchLength", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], TuiTagsInput.prototype, "maxTagsCount", void 0);
__decorate([
    Input("tagSelected"),
    __metadata("design:type", Function)
], TuiTagsInput.prototype, "tagSelectedCb", void 0);
__decorate([
    Input("tagDeselected"),
    __metadata("design:type", Function)
], TuiTagsInput.prototype, "tagDeselectedCb", void 0);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], TuiTagsInput.prototype, "placeholder", null);
__decorate([
    Input(),
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [Array])
], TuiTagsInput.prototype, "options", null);
__decorate([
    Input(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function])
], TuiTagsInput.prototype, "optionsFilter", null);
__decorate([
    Input(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function])
], TuiTagsInput.prototype, "optionsLookup", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], TuiTagsInput.prototype, "optionsField", null);
__decorate([
    Input(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function])
], TuiTagsInput.prototype, "resultFormatter", null);
__decorate([
    Input(),
    __metadata("design:type", TemplateRef)
], TuiTagsInput.prototype, "resultTemplate", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], TuiTagsInput.prototype, "retainSelectedResult", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [Number])
], TuiTagsInput.prototype, "searchDelay", null);
__decorate([
    HostBinding("class.loading"),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TuiTagsInput.prototype, "isSearching", null);
__decorate([
    Input(),
    __metadata("design:type", Number)
], TuiTagsInput.prototype, "maxResults", void 0);
__decorate([
    Output("resultSelected"),
    __metadata("design:type", EventEmitter)
], TuiTagsInput.prototype, "onResultSelected", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], TuiTagsInput.prototype, "transition", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], TuiTagsInput.prototype, "transitionDuration", void 0);
__decorate([
    HostListener("focusin"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TuiTagsInput.prototype, "onFocusIn", null);
__decorate([
    HostListener("focusout", ["$event"]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TuiTagsInput.prototype, "onFocusOut", null);
TuiTagsInput = __decorate([
    Component({
        selector: "tui-tags-input",
        template: "\n        <div class=\"ui input tags-div\" [class.icon]=\"hasIcon\" (click)=\"onClick($event)\">\n            <tui-multi-select-label class=\"tag-label\"\n                                    *ngFor=\"let selected of selectedOptions;\"\n                                    [value]=\"selected\"\n                                    [formatter]=\"configuredFormatter\"\n                                    (deselected)=\"deselectOption($event)\"></tui-multi-select-label>\n            <input class=\"tags-input\"\n                   type=\"text\"\n                   [attr.placeholder]=\"placeholder\"\n                   autocomplete=\"off\"\n                   [(ngModel)]=\"query\"\n                   (paste)=\"onPaste($event)\"\n            >\n            <i *ngIf=\"hasIcon\" class=\"search icon\"></i>\n        </div>\n        <div class=\"results\"\n             suiDropdownMenu\n             [menuTransition]=\"transition\"\n             [menuTransitionDuration]=\"transitionDuration\"\n             menuSelectedItemClass=\"active\">\n\n            <tui-search-result *ngFor=\"let r of results\"\n                               class=\"item\"\n                               [value]=\"r\"\n                               [query]=\"query\"\n                               [formatter]=\"resultFormatter\"\n                               [template]=\"resultTemplate\"\n                               (click)=\"select(r)\"></tui-search-result>\n\n            <div *ngIf=\"results.length == 0\" class=\"message empty\">\n                <div class=\"header\">{{ localeValues.noResults.header }}</div>\n                <div class=\"description\">{{ localeValues.noResults.message }}</div>\n            </div>\n        </div>\n    ",
        styles: ["\n        :host {\n            display: inline-block;\n        }\n\n        /* Fixes positioning when results are pushed above the search. */\n        .results {\n            margin-bottom: .5em;\n        }\n\n        .tags-div {\n            background-color: white;\n            /* This bit sets up the horizontal layout */\n            display: flex;\n            flex-direction: row;\n            /* This bit draws the box around it */\n            border: 1px solid grey;\n            padding: 2px;\n            border-radius: 8px;\n        }\n        .tags-div .tags-input {\n            border: none;\n            outline-offset: 0px !important;\n            outline: none !important;\n        }\n        .tags-div .tags-input input:focus {\n            outline: #0099CC solid 2px;\n        }\n        .tags-div .tag-label {\n            margin: 5px;\n        }\n    "]
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2, TuiLocalizationService])
], TuiTagsInput);
export { TuiTagsInput };
//# sourceMappingURL=tags-input.component.js.map